unit hiComboBox; { ComboBox ver 2.50}

interface

{$I share.inc}

uses Windows,Kol,Share,WinList,Messages;

type
 THIComboBox = class(THIWinList)
   private
     IList:PImageList;
     ICArray:PArray;
     ValArr:PArray;
     Icon:PIcon;
     bmp:PBitmap;
     procedure SetIListProp(Color:TColor);
     procedure SetIcons(const value:PStrListEx);
     procedure SetIndex(const value:PStrListEx);
     procedure _SetIcon(var Item:TData; var Val:TData);
     function _GetIcon(Var Item:TData; var Val:TData):boolean;
     procedure _AddIcon(var Val:TData);
     function  _CountIcon:integer;
     procedure _val_arr_set(var Item:TData; var Val:TData);
     function  _val_arr_get(Var Item:TData; var Val:TData):boolean;
     function  _val_arr_count:integer;
     function  _OnMeasureItem( Sender: PObj; Idx: Integer ):Integer;
     function  _OnDrawItem( Sender: PObj; DC: HDC; const Rect: TRect; ItemIdx: Integer;
                            DrawAction: TDrawAction; ItemState: TDrawState ): Boolean;
   public
     _prop_ReadOnly:byte;
     _prop_Text:string;
     _prop_Strings:string;
     _prop_Sort:boolean;
     _prop_ImgSize:byte;
     _prop_Extended:boolean;
     _prop_CutText:boolean;
     _prop_Gradient:boolean;
     _prop_InversColor:boolean;
     _prop_DepthGradient:integer;
     _prop_Frame:boolean;
     _prop_BumpText:boolean;
     _prop_ColorGradient:TColor;
     _prop_ColorFrame:TColor;
     _prop_BmpTransparent:TColor;
     _prop_IconToBmp:boolean;
     _prop_IdxIcons:PStrListEx;
     _prop_Icons:PStrListEx;

     destructor Destroy; override;
     procedure Init; override;

     procedure _work_doGradient(var _Data:TData; Index:word);
     procedure _work_doInversColor(var _Data:TData; Index:word);
     procedure _work_doDepthGradient(var _Data:TData; Index:word);
     procedure _work_doFrame(var _Data:TData; Index:word);
     procedure _work_doColorGradient(var _Data:TData; Index:word);
     procedure _work_doColorFrame(var _Data:TData; Index:word);
     procedure _work_doBumpText(var _Data:TData; Index:word);
     procedure _work_doCutText(var _Data:TData; Index:word);
     procedure _work_doEditText(var _Data:TData; Index:word);
     procedure _var_EditText(var _Data:TData; Index:word);
     procedure _var_Index(var _Data:TData; Index:word);
     procedure _var_IconArray(var _Data:TData; Index:word);
     procedure _var_ValueArray(var _Data:TData; Index:word);
  end;

function MakeArrayIcon(Names:PChar; Values:array of integer):PStrListEx;

implementation

function MakeArrayIcon;begin Result:= MakeArrayInteger(Names,Values);end;

//-----------------------       ----------------------

type
  TRGB = packed record
    R, G, B, X: Byte;
  end;

  AGRBQuad = array [0..0] of RGBQuad;
  PAGRBQuad = ^AGRBQuad;

  PPoints = ^TPoints;
  TPoints = array[0..0] of TPoint;

function GetRGB(const Color: TColor): TRGB;
var
  iColor: TColor;
begin
  iColor := Color2RGB(Color);
  Result.R := GetRValue(iColor);
  Result.G := GetGValue(iColor);
  Result.B := GetBValue(iColor);
end;

function GetLightColor(Color: TColor; Light: Byte) : TColor;
var
  fFrom: TRGB;
begin
  FFrom := GetRGB(Color);
  Result := RGB(
    Round(FFrom.R + (255 - FFrom.R) * (Light / 100)),
    Round(FFrom.G + (255 - FFrom.G) * (Light / 100)),
    Round(FFrom.B + (255 - FFrom.B) * (Light / 100))
  );
end;

function  GetShadeColor(Color: TColor; Shade: Byte) : TColor;
var
  fFrom: TRGB;
begin
  FFrom := GetRGB(Color);
  Result := RGB(
    Max(0, FFrom.R - Shade),
    Max(0, FFrom.G - Shade),
    Max(0, FFrom.B - Shade)
  );
end;

procedure MyPolyline(DC: HDC;const Points: array of TPoint);
begin
  Polyline(DC, PPoints(@Points)^, High(Points) + 1);
end;

procedure THIComboBox._work_doGradient;begin _prop_Gradient:= ReadBool(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doInversColor;begin _prop_InversColor:= ReadBool(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doDepthGradient;begin _prop_DepthGradient:= ToIntIndex(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doFrame;begin _prop_Frame:= ReadBool(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doColorGradient;begin _prop_ColorGradient:= ToInteger(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doColorFrame;begin _prop_ColorFrame:= ToInteger(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doBumpText;begin _prop_BumpText:= ReadBool(_Data);Control.Invalidate;end;
procedure THIComboBox._work_doCutText;begin _prop_CutText:= ReadBool(_Data);Control.Invalidate;end;

//------------------------       ----------------------

procedure THIComboBox.Init;
var Flags:TComboOptions;
begin
   Flags := [coNoIntegralHeight];
   if (_prop_ReadOnly = 0) or _prop_Extended then include(Flags,coReadOnly);
   if _prop_Sort then include(Flags,coSort);
   if _prop_Extended then include(Flags,coOwnerDrawFixed);
   Control := NewComboBox(FParent,Flags);
   Control.OnSelChange := _OnClick;
   Control.OnMeasureItem:= _OnMeasureItem;
   Control.OnDrawItem:= _OnDrawItem;

inherited;

   Control.Text := _prop_Text;
   SetStrings(_prop_Strings);
   if _prop_Extended then IList:= NewImageList(nil);
   if _prop_Extended and Assigned(_prop_Icons) then SetIcons(_prop_Icons);
   if _prop_Extended and Assigned(_prop_IdxIcons) then SetIndex(_prop_IdxIcons);
end;

destructor THIComboBox.Destroy;
begin
   if Assigned(IList) then free_and_nil(IList);
   If Assigned(Icon) then free_and_nil(Icon);
   If Assigned(bmp) then free_and_nil(bmp);
   if ICArray <> nil then Dispose(ICArray);
   if ValArr <> nil then Dispose(ValArr);
   inherited Destroy;
end;

function THIComboBox._OnMeasureItem;
var   fControl: PControl;
begin
   fControl:= PControl(Sender);
   Result:= max(_prop_ImgSize + 6, fControl.Canvas.TextExtent('W').cy);
end;

//---------------------       -----------------------

function THIComboBox._OnDrawItem;                                    
var   ARect,BRect,cbRect: TRect;
      fControl: PControl;
      _Flags: dword;
      dr,dg,db:byte;
      _Color,ColorDark,ColorLight,lj,dj:TColor;
      t:real;
      j:integer;
begin
   fControl:= PControl(Sender);

   with fControl{$ifndef F_P}^{$endif} do begin
      ARect:= Rect;
      cbRect:= Rect;
      ARect.Left:= ARect.Left + _prop_ImgSize + 8; 
      if (odsComboboxEdit in ItemState) then 
         ARect.Left:= ARect.Left - 3;

      SetBkMode(DC, Font.Color);

      if (odsSelected in ItemState) then begin
         ColorLight:= GetLightColor(_prop_ColorGradient, _prop_DepthGradient);
         Canvas.Brush.Color:= Color2RGB(_prop_ColorFrame);
         if _prop_Frame then FillRect(DC,cbRect,Canvas.Brush.Handle);
         if not (odsComboboxEdit in ItemState) and _prop_Gradient then begin
            ColorDark:= _prop_ColorGradient;
            lj := ToSysColor(ColorLight);
            dj := ToSysColor(ColorDark);
            dr := TRGB(lj).r - TRGB(dj).r;
            dg := TRGB(lj).g - TRGB(dj).g;
            db := TRGB(lj).b - TRGB(dj).b;
            if _prop_Frame then
               for j := 0 to (cbRect.Bottom - cbRect.Top - 3) do begin
                  t := j/(max(cbRect.Bottom - cbRect.Top - 2,2) - 1);
                  with TRGB(dj) do _Color := RGB(Round(dr*t + r),Round(dg*t + g),Round(db*t + b));
                  Canvas.Pen.Color := _Color;
                  SelectObject(DC,Canvas.Pen.Handle);
                  MyPolyline(DC,[MakePoint(cbRect.Left+1,  Rect.Bottom - j - 2),
                             MakePoint(cbRect.Right-1, cbRect.Bottom - j - 2)])
               end
            else
               for j := 1 to (cbRect.Bottom - cbRect.Top) do begin
                  t := j/(max(cbRect.Bottom - cbRect.Top, 1));
                  with TRGB(dj) do _Color := RGB(Round(dr*t + r),Round(dg*t + g),Round(db*t + b));
                  Canvas.Pen.Color := _Color;
                  SelectObject(DC,Canvas.Pen.Handle);
                  MyPolyline(DC,[MakePoint(cbRect.Left,  Rect.Bottom - j),
                             MakePoint(cbRect.Right, cbRect.Bottom - j)]);
               end;
         end
         else begin
            ColorLight:= GetLightColor(_prop_ColorGradient, 30);
            Canvas.Brush.Color:= ColorLight;
            if _prop_Frame then 
               FillRect(DC,MakeRect(cbRect.Left+1,cbRect.Top+1,cbRect.Right-1,cbRect.Bottom-1), Canvas.Brush.Handle)
            else
               FillRect(DC,cbRect,Canvas.Brush.Handle)         
         end;
      end
      else begin
         Canvas.Brush.Color:= Color;
         SetTextColor(DC,Color2RGB(Font.Color));
         FillRect(DC,cbRect,Canvas.Brush.Handle);         
      end;
      if _prop_CutText then
         _Flags:=  DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX or DT_WORD_ELLIPSIS
      else
         _Flags:=  DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX;    
      ARect.Left:= ARect.Left + 4;
      BRect:= ARect; 
      with BRect do begin
         inc(Left);
         inc(Top);
         inc(Right);
         inc(Bottom);
      end;
      if _prop_BumpText and _prop_Gradient then begin
         if not (odsComboboxEdit in ItemState) and (odsSelected in ItemState) then begin
            if _prop_InversColor then
               SetTextColor(DC,GetShadeColor(Font.Color, 100))
            else
               SetTextColor(DC,Color2RGB(clHighlightText));
            DrawText(DC, PChar(Items[ItemIdx]), -1, BRect, _Flags);
            if _prop_InversColor then
               SetTextColor(DC,Color2RGB(clHighlightText))
            else
               SetTextColor(DC,Color2RGB(Font.Color));
         end
         else if (odsSelected in ItemState) then 
            if _prop_InversColor then
               SetTextColor(DC,Color2RGB(clHighlightText))
            else
               SetTextColor(DC,Color2RGB(Font.Color))
         else
            SetTextColor(DC,Color2RGB(Font.Color))
      end
      else
         if (odsSelected in ItemState) then begin
            if _prop_InversColor then
               SetTextColor(DC,Color2RGB(clHighlightText))
            else
               SetTextColor(DC,Color2RGB(Font.Color));
         end
         else
            SetTextColor(DC,Color2RGB(Font.Color));

      DrawText(DC, PChar(Items[ItemIdx]), -1, ARect, _Flags);
      Result:= True;

      if not Assigned(IList) then exit;      

      with cbRect do begin
         Top:= Top + (Bottom - Top - _prop_ImgSize) div 2;
         Left:= 6;
         Bottom:= Top + _prop_ImgSize;
         Right:= Left + _prop_ImgSize;
      end;
      IList.DrawingStyle:= [dsTransparent];      
      IList.StretchDraw(ItemData[ItemIdx],DC,cbRect);
   end;
end;

procedure THIComboBox._work_doEditText;
begin
   Control.Caption := ToString(_Data);
end;

procedure THIComboBox._var_EditText;
begin
   dtString(_Data,Control.Caption);
end;

procedure THIComboBox._var_Index;
begin
   dtInteger(_Data,Control.CurIndex);
end;

procedure THIComboBox.SetIListProp;
begin
   IList.BkColor:= Color;
   IList.Colors:= ilcColor32;
   IList.ImgWidth:= _prop_ImgSize;
   IList.ImgHeight:= _prop_ImgSize;
end;

procedure THIComboBox.SetIcons;
var i:integer;
begin
   if _prop_Imgsize = 0 then _prop_Imgsize:= GetSystemMetrics(SM_CXICON);
   SetIListProp(clWindow);
   for i:= 0 to Value.Count - 1 do IList.AddIcon(Value.Objects[i]);
end;

procedure THIComboBox.SetIndex;
var i:integer;
begin
   for i:= 0 to min((Value.Count - 1),(Control.Count - 1)) do Control.ItemData[i]:= integer(Value.Objects[i]);
end;

//-------------------         ---------------------

procedure THIComboBox._var_IconArray;
begin
  if Assigned(Ilist) then
     begin
        if not Assigned(ICArray) then ICArray := CreateArray(_SetIcon,_GetIcon,_CountIcon,_AddIcon);
        dtArray(_Data,ICArray);
     end
  else dtArray(_Data,nil);
end;

procedure THIComboBox._SetIcon;
var   ind:integer;
begin
   if not Assigned(Ilist) then Exit;
   SetIListProp(clWindow);
   ind:= ToIntIndex(Item);
   if (ind >= 0)and(ind < IList.Count) and _IsIcon(Val) then
      IList.ReplaceIcon(ind, ToIcon(val).handle);
   Control.Invalidate;
end;

function THIComboBox._GetIcon;
var   ind:integer;
begin
   Result:= False;
   if not Assigned(Ilist) then Exit;
   SetIListProp(clWindow);
   ind:= ToIntIndex(Item);
   if (ind >= 0)and(ind < IList.Count) then begin
      if Assigned(Icon) then free_and_nil(Icon);
      Icon:= NewIcon;
      Icon.Handle:= IList.ExtractIcon(ind);
      if not _prop_IconToBmp then
         dtIcon(Val,Icon)
      else begin
         if Assigned(bmp) then free_and_nil(bmp);
         bmp:= NewDIBBitmap(Icon.Size,Icon.Size,pf32bit);
         bmp.Handle:= Icon.Convert2Bitmap(_prop_BmpTransparent);
         dtBitmap(Val,bmp);
      end;
      Result:= True;
   end;
end;

function THIComboBox._CountIcon;
begin
   Result:=0;
   if not Assigned(Ilist) then Exit;
   Result:= IList.Count;
end;

procedure THIComboBox._AddIcon;
begin
   if not Assigned(Ilist) then Exit;
   SetIListProp(clWindow);
   if _IsIcon(Val) then IList.AddIcon(ToIcon(Val).Handle);
   Control.Invalidate;
end;

//------------------         --------------------

procedure THIComboBox._var_ValueArray;
begin
  if ValArr = nil then
    ValArr := CreateArray(_val_arr_set,_val_arr_get,_val_arr_count,nil);
  dtArray(_Data,ValArr);
end;

procedure THIComboBox._val_arr_set;
var ind:integer;
begin
  ind := ToIntIndex(Item);
  if(ind >=0 )and(ind < Control.Count)then
    Control.ItemData[ind] := ToInteger(Val);
end;

function THIComboBox._val_arr_get;
var ind:integer;
begin
  ind := ToIntIndex(Item);
  Result := (ind >=0 )and(ind < Control.Count);
  if Result then
    dtInteger(Val,Control.ItemData[ind]);
end;

function THIComboBox._val_arr_count;
begin
  Result := Control.Count;
end;

end.